
function r = ctc(data,varargin)

%   Cross-trial correlation.
%   
%   CTC(DATA) calculates and displays cross-trial correlation on DATA as 
%   presented in the paper "Cross-trial correlation analysis of evoked 
%   potentials reveals arousal related-attenuation of thalamo-cortical 
%   coupling" published in the Journal of Computational Neuroscience[1].
%   CTC uses other three functions for normalization and visualization of
%   data: CTCVIEWER, MINMAXNORM and PIN2ZERO, all provided in the package.
%   
%   DATA should be an E by N by S array containing N evoked potentials 
%   (number of trials=N) of equal length E each (epochs of E samples) 
%   recorded from S sites/electrodes (S may be 1 or more; S=1 returns auto 
%   cross-trial correlation).
%   
%   CTC(DATA,ALPHA) uses ALPHA to set maximum p-values for correlation 
%   coefficients to be significant - default is 0.05 (p-value is the 
%   probability of getting a correlation as large as the observed value by 
%   random chance, when the true correlation is zero). Insignificant pixels 
%   (correlation coefficients with p-value above ALPHA) are returned as 
%   zeros. 
%   
%   CTC(DATA,ALPHA,STIM) sets the moment of stimulation to STIM. STIM 
%   should be provided in samples from the beginning of the epoch E. For 
%   example, if DATA contains evoked potentials epochs of 200 samples, with 
%   the stimulation at the 50th sample from the beginning of the epoch, 50 
%   should be passed. STIM may also be passed in seconds instead of samples. 
%   For example if DATA contains evoked potentials epochs of 2 seconds, 
%   with the stimulation at 0.5 second from the beginning of the epoch, 0.5 
%   should be passed. However, in this case sampling rate of DATA should 
%   also be provided (see below). If STIM is not provided, CTC assumes that
%   stimulation was at the first sample. CTC(DATA,[],[]) is the same as 
%   CTC(DATA). The value provided by STIM is used to define the baseline 
%   of the DATA epochs (1:STIM) and set its (mean) value to zero prior to
%   correlation calculations.  
%
%   CTC(DATA,ALPHA,STIM,RATE) uses RATE as rate at which the DATA is 
%   sampled (digitized) in Hertz. If RATE is passed, STIM should be passed 
%   in seconds, and not samples (see above). CTC(DATA,[],[],[]) is the 
%   same as CTC(DATA,ALPHA).
%
%   CTC(DATA,ALPHA,STIM,RATE,SMOOTHING) turns off/on gaussian smoohing of 
%   the CTC matrix. Pass string 'off' to turn smoothing off, or 'on' to 
%   turn it on. By default, it is turned off: CTC(DATA,[],[],[],'off') is  
%   the same as CTC(DATA,ALPHA). Smoothing affects only the display of the 
%   results. 
%   
%   CTC results, i.e the correlation matrix, is displayed on one color-
%   scaled image. The corrlation matrix can be ploted raw or smoothed 
%   according to SMOOTHING parameter.  The average evoked potentials from 
%   all sites (electrodes) are normalized (MINMAXNORM) and plotted below 
%   and to the side of the correlation matrix for ease of orientation.  
%
%   R = CTC(...) additionally returns the cross-trial correlation matrix
%   


%   References:
%      [1] A. Sobolewski, E. Kublik, D.A. wiejkowski, S. Leski, J.K. 
%      Kaminski, A. Wrobel, "Cross-trial correlation analysis of evoked 
%      potentials reveals arousal related-attenuation of thalamo-cortical 
%      coupling", Journal of Computational Neuroscience, 2010.
%   
%   Copyright 2009 Aleksander Sobolewski.
%
%   See also (other m-files in the package): CTCVIEWER, MINMAXNORM, PIN2ZERO


if nargin==1 || (nargin>1 && isempty(varargin{1})); alpha = 0.05; else alpha = varargin{1}; end
if nargin>2 && ~isempty(varargin{2}); stim = varargin{2}; else stim = 1; end
if nargin>3 && ~isempty(varargin{3}); rate = varargin{3}; else rate = 1; end

if stim<=0; data = pin2zero(data,1); elseif stim*rate>size(data,1); data = pin2zero(data,1:size(data,1)); else data = pin2zero(data,1:round(stim*rate)); end

data = single(data);

[r,p] = corrcoef(reshape(permute(data,[2 1 3]),size(data,2),size(data,1)*size(data,3)),'rows','complete');
r = r.*(p<alpha);

if nargin>4 && strcmp('on',varargin{4}); 
kernelsize = min([10 round(size(data,1)/20)]);
if kernelsize>0
    kernel = gausswin(kernelsize);
    kernel = kernel*kernel';
    kernel = kernel/sum(sum(kernel));
    r = filter2(kernel,r);
end
end

ctcviewer(r,data,stim,rate);

