
function ax = ctcviewer(r,data,varargin)

%   View results of cross-trial correlation (CTC).
%   
%   CTCVIEWER(R,DATA) displays results of cross-trial correlation as 
%   presented in the paper "Cross-trial correlation analysis of evoked 
%   potentials reveals arousal related-attenuation of thalamo-cortical 
%   coupling" published in the Journal of Computational Neuroscience 
%   article.
%   
%   DATA should be an E by N by S array containing N evoked potentials 
%   (number of trials=N) of equal length E each (epochs of E samples) 
%   recorded from S sites/electrodes (S may be 1 or more; S=1 returns auto 
%   cross-trial correlation). R should be an E*S by E*S cross-trial 
%   correlation matrix.
%   
%   Note that axes are linked when so zooming or panning on the main image, 
%   the bottom and left EP plots will follow follow automatically.
%   
%   CTCVIEWER (R,DATA,STIM) sets the moment of stimulation to STIM. STIM 
%   should be provided in samples from the beginning of the epoch E. For 
%   example if DATA contains evoked potentials epochs of 200 samples, with 
%   the stimulation at the 50th sample from the beginning of the epoch, 50 
%   should be passed. STIM may also be passed in seconds instead of samples. 
%   For example if DATA contains evoked potentials epochs of 2 seconds, 
%   with the stimulation at 0.5 second from the beginning of the epoch, 0.5 
%   should be passed. However, in this case sampling rate of DATA should 
%   also be provided (see below). If STIM is not provided, CTCVIEWER 
%   assumes that stimulation was at the first sample.  
%   
%   CTCVIEWER (R,DATA,STIM,RATE) uses RATE as rate at which the DATA is 
%   sampled (digitized) in Hertz. If RATE is passed, STIM should be passed 
%   in seconds, and not samples (see above).
%   
%   References:
%      [1] A. Sobolewski, E. Kublik, D.A. wiejkowski, S. Leski, J.K. 
%      Kaminski, A. Wrobel, "Cross-trial correlation analysis of evoked 
%      potentials reveals arousal related-attenuation of thalamo-cortical 
%      coupling", Journal of Computational Neuroscience, 2010.
%   
%   Copyright 2009 Aleksander Sobolewski.
%
%   See also (other m-files in the package): CTC, MINMAXNORM, PIN2ZERO

h = waitbar(1,'message') 

if nargin>2; stim = varargin{1}; else stim = 1; end
if nargin>3; rate = varargin{2}; stim = round(stim*rate); else rate = 1; end

sweep = (1:size(data,1))-stim; 

step = [logspace(1,10,10)/100000 5*logspace(1,10,10)/100000];
[v,idx] = min(abs(step-size(data,1)/rate/5));
step = round(step(idx)*rate);

meaneps = reshape(minmaxnorm(nanmean(data,2),'columns'),size(data,1)*size(data,3),1);
meaneps(1:size(data,1):size(r,1)) = NaN;

% Create figure
figure('Position',[50 50 900 900]);

% Bottom horizontal mean EPs plot
ax(2) = subplot('Position',[0.25 0.15 0.65 0.1]);
plot(meaneps,'k','LineWidth',1)
axis([1 size(data,1)*size(data,3) -0.1 1.1])
line([1:size(data,1):size(r,1); 1:size(data,1):size(r,1)],[-0.1; 1.1],'Color',[0 0 0]) 

% Left verical mean EPs plot
ax(3) = subplot('Position',[0.15 0.25 0.1 0.65]);
plot(1-meaneps,1:size(data,1)*size(data,3),'k','LineWidth',1)
axis([-0.1 1.1 1 size(data,1)*size(data,3)])
line([-0.1; 1.1],[1:size(data,1):size(r,1); 1:size(data,1):size(r,1)],'Color',[0 0 0]) 

% Correlation matrix image
ax(1) = subplot('Position',[0.25 0.25 0.65 0.65]);
imagesc(r,[-0.5 0.5]); 
axis xy; axis([1 size(data,1)*size(data,3) 1 size(data,1)*size(data,3)])
cmap = colormap(jet(256)); 
line([1:size(data,1):size(r,1); 1:size(data,1):size(r,1)],[1; size(r,1)],'Color',[0 0 0]) 
line([1; size(r,1)],[1:size(data,1):size(r,1); 1:size(data,1):size(r,1)],'Color',[0 0 0]) 
line([ones(1,size(data,3)); 1:size(data,1):size(r,1)],[size(r,1):-size(data,1):1; repmat(size(r,1),1,size(data,3))],'Color',[0 0 0]) 
line([size(r,1):-size(data,1):1; repmat(size(r,1),1,size(data,3))],[ones(1,size(data,3)); 1:size(data,1):size(r,1)],'Color',[0 0 0]) 
line([1 size(r,1)],[1 size(r,1)],'Color',[0 0 0]) 

% Axes annotations
ticks = find(round(sweep/step)*step-sweep==0);
%ticks(end) = [];
ticklabels = sweep(ticks)/rate;
sweep = linspace(1,size(data,1),length(sweep));
tickpositions = sweep(ticks);

tickpositions = repmat(tickpositions',1,size(data,3))+repmat(0:size(data,1):size(data,3)*size(data,1)-1,length(tickpositions),1);
tickpositions = tickpositions(:);

set(ax(1),'XTick',tickpositions); set(ax(1),'XTickLabel',[])
set(ax(1),'YTick',tickpositions); set(ax(1),'YTickLabel',[])
set(ax(1),'XGrid','on','XColor',[0 0 0],'YGrid','on','YColor',[0 0 0])
set(ax(2),'XTick',tickpositions); set(ax(2),'XTickLabel',ticklabels,'FontSize',10)
set(ax(2),'YTick',[]); 
set(ax(2),'XGrid','on'); 
set(ax(3),'YTick',tickpositions); set(ax(3),'YTickLabel',ticklabels,'FontSize',10)
set(ax(3),'XTick',[]); 
set(ax(3),'YGrid','on'); 

% Link axes
hlink1 = linkprop(ax(1:2),'XLim');
hlink2 = linkprop(ax([1 3]),'YLim');
hlinks = [hlink1 hlink2];
set(gcf,'userdata',hlinks);




